"""
Classifies: CHEBI:2468 secondary alpha-hydroxy ketone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_secondary_alpha_hydroxy_ketone(smiles: str):
    """
    Determines if a molecule contains a secondary alpha-hydroxy ketone group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains secondary alpha-hydroxy ketone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for secondary alpha-hydroxy ketone:
    # [OH]-[CH]([#6,#7,#8,#9,#15,#16,#17,#35,#53])-C(=O)-[#6]
    # Explicitly defines the non-hydrogen substituent types and requires carbon after carbonyl
    pattern = Chem.MolFromSmarts('[OH][CH]([#6,#7,#8,#9,#15,#16,#17,#35,#53])C(=O)[#6]')
    
    if pattern is None:
        return None, "Invalid SMARTS pattern"
        
    matches = mol.GetSubstructMatches(pattern)
    
    if not matches:
        return False, "No secondary alpha-hydroxy ketone group found"
        
    # Verify each match
    for match in matches:
        # Get the relevant atoms
        oh_oxygen = mol.GetAtomWithIdx(match[0])
        alpha_carbon = mol.GetAtomWithIdx(match[1])
        r_group = mol.GetAtomWithIdx(match[2])
        carbonyl_carbon = mol.GetAtomWithIdx(match[3])
        
        # Check that alpha carbon has exactly one hydrogen
        if alpha_carbon.GetTotalNumHs() != 1:
            continue
            
        # Check that alpha carbon has exactly three neighbors
        if len(list(alpha_carbon.GetNeighbors())) != 3:
            continue
            
        # Check that the carbonyl carbon is not part of an ester, amide, or acid
        is_valid = True
        for neighbor in carbonyl_carbon.GetNeighbors():
            if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                is_valid = False
                break
            if neighbor.GetSymbol() == 'N':
                is_valid = False
                break
                
        if is_valid:
            substituent_type = r_group.GetSymbol()
            return True, f"Contains secondary alpha-hydroxy ketone with {substituent_type} substituent"
            
    return False, "No valid secondary alpha-hydroxy ketone group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:2468',
                          'name': 'secondary alpha-hydroxy ketone',
                          'definition': 'An alpha-hydroxy ketone in which the '
                                        'carbonyl group and the hydroxy group '
                                        'are linked by a carbon bearing one '
                                        'hydrogen and one organyl group. '
                                        'Secondary alpha-hydroxy ketones are '
                                        'also known as acyloins, and are '
                                        'formally derived from reductive '
                                        'coupling of two carboxylic acid '
                                        'groups.',
                          'parents': ['CHEBI:139588', 'CHEBI:35681']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('O=C1C2=C(OC([C@H]1O)(C)C)[C@H](O)[C@]3(O)C[C@@H](O[C@H]3C2)C(O)(C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O1CC(O)C(=O)C(O)=C1C', 'Contains secondary alpha-hydroxy "
               "ketone group'), "
               "('CO[C@@H]([C@@H]1Cc2cc3cc(O[C@H]4C[C@@H](O[C@H]5C[C@@H](O)[C@H](O)[C@@H](C)O5)[C@H](O)[C@@H](C)O4)c(C)c(O)c3c(O)c2C(=O)[C@H]1O[C@H]1C[C@@H](O[C@H]2C[C@@H](O[C@H]3C[C@](C)(O)[C@H](O)[C@@H](C)O3)[C@@H](O)[C@@H](C)O2)[C@H](O)[C@@H](C)O1)C(=O)[C@@H](O)C(C)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('[H][C@]12[C@H](CO)[C@@]1(C)CC[C@@]1(O)[C@]22OC(=O)[C@@]11CCCC(C)(C)[C@]1([H])C(=O)[C@@H]2O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@H](COP([O-])([O-])=O)C([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('S1C2=NC(C1)C(=O)N/C(/C(=O)N3C(C(=O)NC(C(=O)N(C(C(C)C)C(OCC(C(NC(C(NC(C(N4C2CC4C)=O)CC(C)C)=O)C)=O)O)=O)C)C(O)C)CCCC3)=C\\\\C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(OC([C@@H]1O)(C)C)C=C(OC)C(=C2)OC', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('ClC1=C(O)C=CC(=C1)C[C@@H]2N(C(=O)[C@@H](N3C(=O)[C@H](NC(=O)[C@H](CCCN=C(N)N)NC([C@H]([C@H](OC([C@@H](NC2=O)C(CC)C)=O)C)NC(=O)[C@H](O)COS(=O)(=O)O)=O)CC[C@H]3O)C(CC)C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(CC(O)C(O)=O)C=1C(OCC=C)=CC=CC1', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('OC[C@H](O)[C@@H](O)C(=O)[C@H](O)C(O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), ('OC(COP(O)(O)=O)C(O)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('OC[C@@H](O)[C@@H](O)C(=O)[C@@H](O)C(O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('O(P(O)(O)=O)CC([C@H](C(=O)O)O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O=C1NC(=O)C(=C1C2=CC=C(OC[C@H](O)C(=O)OC)C=C2)CC(C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(O)[C@]1(O[C@@H]2[C@@H](O)[C@H](N3C4=NC=NC(=C4N=C3)N)OC2=C1)[C@H](O)C(=O)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@H](C(O)=O)C(=O)S[*]', 'Contains secondary alpha-hydroxy "
               "ketone group'), "
               "('O(CC(O)C(O)=O)C(=O)\\\\C=C\\\\C1=CC(O)=C(OC)C=C1', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('C(C(COP(=O)([O-])[O-])O)([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O=C(NCCCCCCCC/C=C\\\\CCCCCC)[C@H](O)CO[C@H]1O[C@@H]([C@@H](O)[C@@H]([C@H]1NC(=O)C)O)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('OC(C(O)=O)C(=C)OP(O)(O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('CC(C(C(N1C2=CC=CC=C2[C@]34CCN5C[C@]6([C@@H](C)OC[C@]([C@]6(C[C@@]35O)[H])([C@@]41[H])[H])[H])=O)O)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(C[C@@H](C(CO)=O)O)P([O-])(=O)[O-]', 'Contains secondary "
               "alpha-hydroxy ketone group'), ('C([C@@H](C(CO)=O)O)([O-])=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@@H](COP([O-])([O-])=O)[C@@H](O)C(=O)[C@H](O)C([O-])=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C(OC)=C(OC[C@@H]1O)C', 'Contains secondary alpha-hydroxy "
               "ketone group'), "
               "('C1=2N(C3=CC(=CC=C3C=C1C([N-]C(N2)=O)=O)O)C[C@@H]([C@@H]([C@@H](COP(OC[C@H](C([O-])=O)O)(=O)[O-])O)O)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('CO[C@H](CO)CC1O[C@@H]2[C@@H](NC(=O)[C@@H](O)[C@]3(CC(=C)[C@@H](C)[C@@H](C)O3)OC)OCO[C@@H]2[C@@H](OC)C1(C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(OC([C@H]1O)(C)C)C=3C(=O)[C@@H]4[C@H](C)[C@H](C3C(=C2)CC=C(C)C)C4', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(O[C@]([C@@H]1O)([C@H]3OC(=O)CC3)C)C=C(CO)C=C2O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(NCCCCCCCC/C=C\\\\CCCCCCC)[C@H](O)CO[C@H]1O[C@@H]([C@@H](O)[C@@H]([C@H]1NC(=O)C)O)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1O[C@@H]2N([C@@H](/C=C/C)[C@@H]([C@@H](NC3=C(C(=O)O)C=CC=C3)C4=C5OC([C@H](O)C(C5=CC(=C4/C=C/C=C/C)CC=C(C)C)=O)(C)C)C=6C2=C7OC([C@H](O)C(C7=CC6CC=C(C)C)=O)(C)C)C8=C1C=CC=C8', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1OC[C@@]23[C@@]4([C@@]5(O)[C@H](O[C@@H]2[C@@H]([C@](O)(C)CC3)C5)C[C@H]4OC(=O)C=CCCC67[C@@H](C(=C1)CCO6)OC(=O)[C@H]7O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(C[C@H](C(CO)=O)O)P([O-])(=O)[O-]', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O1[C@]23[C@@]([C@@H](OC(=O)C)C[C@]([C@]4([C@@]1(C[C@H](OC(=O)C)C([C@@H]4CC(OC)=O)(C)C)[H])C)(C2=C)[H])([C@H](OC(=O)[C@H]3O)C=5C=COC5)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('OC(C([O-])=O)C(=O)CCC([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), ('OC(C(=O)CCC(O)=O)C(=O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('[H][C@@]1(OC(=O)[C@@H](O)C1=O)[C@@H](O)CO', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('OCC([C@@H](C(O)=O)O)=O', 'Contains secondary alpha-hydroxy "
               "ketone group'), "
               "('O=C1O[C@H](CCC[C@@H]([C@]2(C(=O)[C@H](OC3=C(C(O)=CC(=C3)C)C)[C@](OC)(C)[C@@H](C2=O)O)C)CC(C4=C(C1)C=C(O)C=C4O)=O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OCC(O)C(=O)N[C@@H]1C(=O)NC(C(=O)NC2C(=O)N([C@@H](CC3=CC=CC=C3)C(N(C(C(NC(C(O[C@@H]1C)=O)C(C)C)=O)CC4=CC=C(O)C=C4)C)=O)C(O)CC2)CCCN=C(N)N)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(O)[C@H](O)[C@]1(O[C@H]([C@@](OC)(C)[C@H]([C@@H]1C)OC)C[C@H]2O[C@]3(O[C@@]([C@@H]4O[C@H]([C@@H]5O[C@@H]([C@H]6O[C@](O)([C@H](C)[C@H]([C@@H]6C)O[C@@H]7O[C@@H]([C@@H](OC)CC7)C)C)CC5)CC4)(C)[C@@H]([C@H]3C)OC)[C@H](C)[C@@H](C2)OC)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1N(C=2C=CC=C3C2[C@@]14O[C@]5(O)[C@@H](O)C(=O)N6[C@@]5([C@@H]4[C@H](C6(C)C)C3)O)CCCC(=O)OC', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(N1[C@H]([C@H](O)CC(=O)N)CCC1)C([C@H](O)C(/C=C/NC(=O)[C@H](O)[C@@]2(O[C@@H]([C@H](C)C(C2)=C)C)OC)CO)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(C(=O)CCCCCCCC/C=C\\\\CCC)C[C@H](O)C(O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('OCC([C@@H](C([O-])=O)O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('P(OCC(O)C([O-])=O)([O-])(=O)O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O=C1O[C@H](CCC=CC=CCC[C@H](O)CC(O)C[C@H]2O[C@@](O)(C(O)C(=O)O[C@H](CCC=CC=CCC[C@@H](CC(C[C@H]3O[C@](C1O)(O)[C@@H](CC3)C)=O)O)C)[C@H](C)CC2)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OCC(O)C(=O)NC1C(=O)NC(C(=O)NC2C(=O)N(C(CC(C)C)C(N(C(C(NC(C(OC1C)=O)C(C)C)=O)CC3=CC=CC=C3)C)=O)C(O)CC2)CCCN=C(N)N)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O1[C@]2(O)[C@@H](CC[C@]1(C(C(=O)CC(O)CCC=CC=CCCC(OC(=O)C(O)[C@@]3(O[C@@](CC[C@H]3C)(C(C(=O)CC(O)CCC=CC=CCCC(OC(=O)C2O)C)C)[H])O)C)C)[H])C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1O[C@H]2[C@@]3([C@@]4([C@@H](C=C(C)CC4)OC(C2)C35OC5)COC(=O)C6[C@@]7([C@@H]8[C@](C=CC=C1)(OCC7)C(O)C(=O)O8)O6)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('N([C@@H](CCCC[NH2+]CC(=O)[C@H](O)COP([O-])(=O)[O-])C(*)=O)*', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1O[C@H](CCC=CC=CCC[C@H](O)CC(=O)C[C@H]2O[C@@](O)(C(O)C(=O)O[C@H](CCC=CC=CCC[C@@H](CC(C[C@H]3O[C@](C1O)(O)[C@@H](CC3)C)=O)O)C)[C@H](C)CC2)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1N(C=2C=CC=C3C2[C@@]14O[C@]5(O)[C@@H](O)C(=O)N6[C@@]5([C@@H]4[C@H](C6(C)C)C3)O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@@H](COP(O)(O)=O)[C@@H](O)C(=O)[C@H](O)C(O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('OC(C([O-])=O)C(=O)C([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), ('C([C@@H](C(CO)=O)O)(O)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O1C(C(O)C(=O)C2=C1C=CC(OC)=C2)(C)C', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O=C(NCCCCCCCCCCCCCCCCC)[C@H](O)CO[C@H]1O[C@@H]([C@@H](O)[C@@H]([C@H]1NC(=O)C)O)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('CO[C@@H]1[C@H]2OCO[C@H](NC(=O)[C@@H](O)[C@]3(CC(=C)[C@@H](C)[C@@H](C)O3)OC)[C@H]2O[C@H](C[C@H](O)CCC\\\\C=C\\\\C=C\\\\C=C\\\\C(=O)NC(CCCNC(N)=N)C(O)=O)C1(C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1N2[C@@]3(O[C@@]([C@@H]1O)(O)C)C(=O)O[C@]45[C@H]3[C@H](C2(C)C)CC=6C=CC=C(C46)N(C5=O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('P(=O)([O-])(OC[C@@H](O)C(=O)O[As](=O)([O-])[O-])[O-]', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OC[C@@H](O)C(=O)N[C@@H]1C(=O)N[C@H](C(=O)N[C@@H]2C(=O)N([C@@H]([C@H](CC)C)C(N([C@H](C(N[C@H](C(O[C@@H]1C)=O)[C@H](CC)C)=O)CC3=CC=CC=C3)C)=O)[C@H](OC)CC2)CCCN=C(N)N)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OC[C@@H](O)C(=O)N[C@@H](C(=O)N[C@@H]1C(=O)N[C@H](C(=O)N[C@@H]2C(=O)N([C@@H](CC(C)C)C(N([C@H](C(N[C@H](C(O[C@@H]1C)=O)C(C)C)=O)CC3=CC=C(O)C=C3)C)=O)[C@H](O)CC2)CCCN=C(N)N)CC(C)C)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(O)[C@H](O)[C@]1(O[C@H]([C@@H](C)C([C@@H]1C)OC)C[C@H]2O[C@]3(OC([C@@H]4O[C@H]([C@@H]5O[C@@H]([C@H]6O[C@](O)([C@H](C)C[C@@H]6C)C)CC5)C[C@H]4O[C@@H]7O[C@@H]([C@@H](OC)CC7)C)(C)C[C@H]3C)[C@H](C)[C@@H](C2)OC)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C([C@H](O)[C@]1(O[C@H]([C@@](OC)(C)[C@H]([C@@H]1C)OC)C[C@H]2O[C@]3(O[C@@]([C@@H]4O[C@H]([C@@H]5O[C@@H]([C@H]6O[C@](OC)([C@H](C)[C@H]([C@@H]6C)O[C@@H]7O[C@@H]([C@@H](OC)CC7)C)C)CC5)CC4)(C)[C@@H]([C@H]3C)OC)[C@H](C)[C@@H](C2)OC)O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O1C23C(C(OC(=O)C)CC(C4(C1CC(OC(=O)C)C(C4C(O)C(OC)=O)(C)C)C)C2=C)(C(OC(=O)C3O)C=5C=COC5)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(OC([C@@H]1O)(C)C)C(=C(\\\\C=C/C=C/C)C(=C2)CC=C(C)C)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(C1=NC=C(C)C2=C1NC=3C=CC=CC23)C(O)CO[C@@H]4O[C@H]([C@H](O)[C@H]([C@H]4O)O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OC[C@@H](O)C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@@H]1C(=O)N/C(/C(=O)N[C@@H]2C(=O)N([C@@H](CC3=CC=CC=C3)C(N([C@H](C(N[C@H](C(O[C@@H]1C)=O)C(C)C)=O)CC4=CC=C(O)C=C4)C)=O)[C@H](OC)CC2)=C\\\\C)CCC5=CC=C(O)C=C5)C)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('C(=O)(COP(=O)(O)O)[C@@H](C(O)=O)O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O(CC(O)C(O)=O)C=1C(OC)=CC=CC1', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OCC(O)C(=O)N[C@@H]1C(=O)N[C@H](C(=O)N[C@@H]2C(=O)N([C@@H]([C@H](CC)C)C(N([C@H](C(N[C@H](C(O[C@@H]1C)=O)[C@H](CC)C)=O)CC3=CC=CC=C3)C)=O)[C@H](O)CC2)CCCCN(C)C)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(NCCCCCCCCCCCCCCCC)[C@H](O)CO[C@H]1O[C@@H]([C@@H](O)[C@@H]([C@H]1NC(=O)C)O)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1OC[C@@]23[C@@]4([C@@]5(OC5)[C@H](O[C@@H]2C=C(C)CC3)C[C@H]4OC(=O)C=CC=CC67C(C(=C1)CCO6)OC(=O)C7O)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(OCC(O)C(=O)NC(C(=O)OC(C(C(=O)NC(C(=O)NO)CCCN(O)C(=N)N)C)CCCCCCCCCCC)CCCN(O)C=O)C1=C(O)C=CC=C1', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(OC([C@@H]1O)(C)C)C=3C(=O)[C@@H]4[C@H](C)[C@H](C3C(=C2)CC=C(C)C)C4', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(OC[C@@H](O)C(=O)N[C@@H](C(=O)O)CC1=CC=C(O)C=C1)[C@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)C[C@H](N)CCCCCC(C)C)CC(=O)N)[C@@H](O)C(=O)N)[C@@H](O)C2=CC=CC=C2', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C(NCC(=O)N)[C@H]1N(C(=O)C([C@H](O)C(/C=C/NC(=O)[C@@H](O)[C@@]2(O[C@@H]([C@H](C)C(C2)=C)C)O)C)C)CCC1', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP([O-])(=O)OP([O-])(=O)OCC(O)C([O-])=O)[C@@H](O)[C@H]1O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@H]1COc2cc(O)ccc2C1=O', 'Contains secondary alpha-hydroxy "
               "ketone group'), ('OC(C([O-])=O)C([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), ('O[C@H](COP(O)(O)=O)C(O)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1O[C@H](CCC=CC=CCC[C@H](O)CC(O)C[C@H]2O[C@@](O)(C(O)C(=O)O[C@H](CCC=CC=CCC[C@@H](CC(C[C@H]3O[C@](C1O)(O)[C@@H](CC3)C)O)O)C)[C@H](C)CC2)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(C1(C(O)C(OC2=C(C(O)=C(C(=C2)C)C(OC)=O)C)=C(C(=O)C1O)C)C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1O[C@@H](C[C@H]2O[C@@H]3[C@H](OCO[C@@H]3[C@H](C2(C)C)OC)NC(=O)[C@H](O)[C@@]4(O[C@@H]([C@H](C)C(C4)=C)C)OC)CC1', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('CCC(C)C1NC(=O)C(Cc2ccccc2)N(C)C(=O)C(C(C)CC)N2C(O)CCC(NC(=O)C(CCCNC(N)=N)NC(=O)C(NC(=O)C(O)COS(O)(=O)=O)C(C)OC1=O)C2=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('CC(=O)[C@@H](O)COP([O-])([O-])=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('CC1(C)CC[C@@]2(CC[C@]3(C)C(=CC[C@@H]4[C@@]5(C)CC[C@H](O[C@@H]6OC[C@@H](O)C(=O)[C@H]6O[C@@H]6O[C@H](CO)[C@@H](O)[C@H](O)[C@H]6O)C(C)(C)[C@@H]5CC[C@@]34C)[C@@H]2C1)C(O)=O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('S(=O)(=O)(OC[C@@H](O)C(=O)N[C@H](C(=O)N[C@@H]1C(=O)N[C@H](C(=O)N[C@@H]2C(=O)N([C@@H]([C@H](O)C)C(N([C@H](C(N[C@H](C(O[C@@H]1C)=O)[C@@H](CC)C)=O)CC3=CC=C(O)C=C3)C)=O)[C@H](O)CC2)CCCN=C(N)N)CC4=CC=C(O)C=C4)O', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('C1([C@@H]([C@H]([C@@H](C([C@@H]1O)=O)O)O)O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('O(P([O-])([O-])=O)CC([C@H](C(=O)[O-])O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('O[C@H](COP([O-])([O-])=O)C(=O)OP([O-])([O-])=O', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('BrC1=C(C=C(O)C=C1)[C@@H](OC)CC[C@@H]([C@H]2O[C@@]34OC(=C(C)CC3(C)C)C(O)C(=O)O[C@H](CC(O[C@H]([C@@H]2C)C4)=O)[C@H](O)C)C', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('CSC(=O)[C@H](O)COP(O)(O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('O=C(OCC(O)C(=O)NC(C(=O)OC(C(C(=O)NC(C(=O)NO)CCCN(O)C(=N)N)C)CCCCCCCC)CCCN(O)C=O)C1=C(O)C=CC=C1', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O=C1OC2(C(=CC(C2O)=O)C)C=3C1=C(O)C=C(OC)C3', 'Contains "
               "secondary alpha-hydroxy ketone group'), "
               "('O=C1C2=C(OC([C@@H]1O)(C)C)C(=C(/C=C/C=C/C)C(=C2)CC=C(C)C)CO', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O[C@H](COP(O)(O)=O)C(=O)OP(O)(O)=O', 'Contains secondary "
               "alpha-hydroxy ketone group'), "
               "('ClC1=CC=C(C[C@@H]2N3C(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)C(O)COS(=O)(=O)O)[C@@H](OC(C(NC([C@@H](N(C2=O)C)CC4=CC=C(O)C=C4)=O)C(CC)C)=O)C)CCCN=C(N)N)CC[C@H]3O)C=C1', "
               "'Contains secondary alpha-hydroxy ketone group'), "
               "('O(P([O-])([O-])=O)CC([C@@H](C(=O)[O-])O)=O', 'Contains "
               "secondary alpha-hydroxy ketone group')]\n"
               'False negatives: '
               "[('C[C@@H]1O[C@H](OP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2O)n2ccc(=O)[nH]c2=O)[C@H](NC(C)=O)[C@H](O)C1=O', "
               "'No secondary alpha-hydroxy ketone group found'), "
               "('COC1=C(O)C=CC(=C1)C1OC2=CC(O)=CC(O)=C2C(=O)C1O', 'No "
               "secondary alpha-hydroxy ketone group found'), "
               "('[H][C@@]12C[C@H](O)C(=O)C(C)(C)C1=CC[C@@]1([H])[C@]3(C)C[C@@H](O)[C@]([H])([C@@](C)(O)C(=O)CCC(C)(C)OC(C)=O)[C@@]3(C)CC(=O)[C@@]21C', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('[H][C@]1(Cc2cc3cc(O[C@H]4C[C@@H](O[C@@H]5C[C@@H](O)[C@@H](OC)[C@@H](C)O5)[C@@H](OC(C)=O)[C@@H](C)O4)cc(O)c3c(O)c2C(=O)[C@H]1O[C@H]1C[C@@H](O[C@H]2C[C@@H](O[C@H]3C[C@](C)(O)[C@@H](OC(=O)C(C)C)[C@H](C)O3)[C@H](O)[C@@H](C)O2)[C@H](O)[C@@H](C)O1)[C@H](OC)C(=O)[C@@H](O)[C@@H](C)O', "
               "'No secondary alpha-hydroxy ketone group found'), "
               "('O[C@H]1[C@@H](OC2=C(C=CC(O)=C2)C1=O)C1=CC=C(O)C(O)=C1', 'No "
               "secondary alpha-hydroxy ketone group found'), "
               "('O[C@H]1[C@@H](Oc2cc(O)cc(O)c2C1=O)c1ccc(O)c(O)c1', 'No "
               "secondary alpha-hydroxy ketone group found'), "
               "('CC1(C)[C@@H](O)C(=O)c2c1[nH]c1cc(O)ccc21', 'No valid "
               "secondary alpha-hydroxy ketone group found'), "
               "('[H][C@@]12C[C@@H](O)C(=O)[C@@]1(C)CC[C@]1([H])C3=C(CC[C@@]21[H])C=C(O)C=C3', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('O=C1C=C(N)C[C@@]1(O)C=C', 'No secondary alpha-hydroxy ketone "
               "group found'), "
               "('[H][C@@]12C[C@@]3([H])C(C)=CC(=O)[C@@H](O)[C@]3(C)[C@@]3([H])[C@@H](O)[C@H](O)[C@@]4(C)OC[C@@]13[C@@]4([H])[C@@H](OC(=O)C=C(C)C)C(=O)O2', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('C[C@H]1C\\\\C=C/C(=O)[C@@H](O)[C@@H](O)C\\\\C=C\\\\c2cc(O)cc(O)c2C(=O)O1', "
               "'No secondary alpha-hydroxy ketone group found'), "
               "('[H][C@@]12C=C(O)C(=O)C(C)(C)C1=CC[C@@]1([H])[C@]3(C)C[C@@H](O)[C@]([H])([C@@](C)(O)C(=O)CCC(C)(C)OC(C)=O)[C@@]3(C)CC(=O)[C@@]21C', "
               "'No secondary alpha-hydroxy ketone group found'), "
               "('O=C([C@@]1([C@@H]2C(=C)CC[C@]2(O)C1)C)[C@H](O)C=C(C)C', 'No "
               "secondary alpha-hydroxy ketone group found'), "
               "('CCCCCCCCCC\\\\C=C(/[C@H](O)C(C)=O)C(=O)OC', 'No valid "
               "secondary alpha-hydroxy ketone group found'), "
               "('C[C@@]1([C@H]2[C@H](C(O)=O)[C@@]34CC(=C)[C@@](O)(C3)CC[C@H]4C2=CC(=O)[C@@H]1O)C(O)=O', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('C[C@H]1[C@H]2[C@H](Cc3c[nH]c4ccccc34)NC(=O)[C@@]22[C@@H](\\\\C=C\\\\C[C@H](C)\\\\C=C(C)\\\\[C@@H](O)C(=O)\\\\C=C\\\\C2=O)[C@H](O)C1=C', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('OC1C(OC2=C(C=CC(O)=C2)C1=O)C1=CC=C(O)C(O)=C1', 'No secondary "
               "alpha-hydroxy ketone group found'), ('OCC(O)C(=O)COP(O)(O)=O', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('[H][C@@]1(Oc2cc(O)cc(O)c2C(=O)[C@@H]1O)c1cc(O)c2O[C@]([H])([C@H](CO)c2c1)c1ccc(O)c(OC)c1', "
               "'No secondary alpha-hydroxy ketone group found'), "
               "('OC[C@@H](O)C(=O)CO', 'No valid secondary alpha-hydroxy "
               "ketone group found'), "
               "('O.O.O.[H][C@@]1(C[C@@]2(O)[C@@H](OC(=O)c3ccccc3)[C@]3([H])[C@@]4(CO[C@@H]4C[C@H](O)[C@@]3(C)C(=O)[C@H](O)C(=C1C)C2(C)C)OC(C)=O)OC(=O)[C@H](O)[C@@H](NC(=O)OC(C)(C)C)c1ccccc1', "
               "'No valid secondary alpha-hydroxy ketone group found'), "
               "('OC1(C(O)=C(C(O)=C(C1=O)C(=O)C(C)C)CC=C(C)C)CC=C(C)C', 'No "
               "secondary alpha-hydroxy ketone group found'), "
               "('C=1C=CC=CC1C(C(C)=O)O', 'No valid secondary alpha-hydroxy "
               "ketone group found'), ('BrC=1C(=O)[C@](O)(C=C)CC1N', 'No "
               "secondary alpha-hydroxy ketone group found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 20,
    'num_false_positives': 100,
    'num_true_negatives': 13248,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.16666666666666666,
    'recall': 0.8333333333333334,
    'f1': 0.2777777777777778,
    'accuracy': 0.9922225545916841}


"""
Classifies: CHEBI:136528 dihydroxydocosahexaenoate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_dihydroxydocosahexaenoate(smiles: str):
    """
    Determines if a molecule is a dihydroxydocosahexaenoate, i.e., a hydroxy polyunsaturated fatty acid anion
    obtained by the deprotonation of the carboxy group of any dihydroxydocosahexaenoic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a dihydroxydocosahexaenoate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for C22 molecule
    if mol.GetNumAtoms() != 22:
        return False, "The molecule is not a C22 compound"

    # Check for anion
    formal_charge = Descriptors.FormalCharge(mol)
    if formal_charge != -1:
        return False, "The molecule is not an anion"

    # Check for carboxylate group
    carboxylate_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetFormalCharge() == 0 and atom.GetTotalNumHs() == 0:
            neighbors = [mol.GetAtomWithIdx(neighbor_idx) for neighbor_idx in atom.GetNeighbors()]
            if len(neighbors) == 3 and all(neighbor.GetSymbol() == 'O' for neighbor in neighbors):
                carboxylate_found = True
                break

    if not carboxylate_found:
        return False, "No carboxylate group found"

    # Check for six double bonds
    num_double_bonds = Chem.rdMolDescriptors.CalcNumAromaticRings(mol)
    if num_double_bonds != 6:
        return False, "The molecule does not have six double bonds"

    # Check for two hydroxy groups
    num_hydroxy_groups = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1:
            num_hydroxy_groups += 1

    if num_hydroxy_groups != 2:
        return False, "The molecule does not have two hydroxy groups"

    return True, "The molecule is a dihydroxydocosahexaenoate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:136528',
                          'name': 'dihydroxydocosahexaenoate',
                          'definition': 'A hydroxy polyunsaturated fatty acid '
                                        'anion obtained by the deprotonation '
                                        'of the carboxy group of any '
                                        'dihydroxydocosahexaenoic acid.',
                          'parents': ['CHEBI:131864', 'CHEBI:131871']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute 'FormalCharge'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:25095 L-lysine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdFMCS

def is_L_lysine_derivative(smiles: str):
    """
    Determines if a molecule is an L-lysine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an L-lysine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # L-lysine core structure SMILES (without H's)
    l_lysine = Chem.MolFromSmiles("N[C@@H](CCCCN)C(=O)O")
    
    if l_lysine is None:
        return None, None
    
    # Find maximum common substructure between input and L-lysine
    mcs = rdFMCS.FindMCS([mol, l_lysine], 
                        atomCompare=rdFMCS.AtomCompare.CompareElements,
                        bondCompare=rdFMCS.BondCompare.CompareOrder,
                        matchChiralTag=True,
                        ringMatchesRingOnly=True,
                        completeRingsOnly=True)
    
    if mcs.numAtoms < 7:  # Must contain most of L-lysine backbone
        return False, "Does not contain L-lysine core structure"
        
    # Check if the molecule has the L-lysine chiral center preserved
    smart_pattern = Chem.MolFromSmarts("[NH2][C@@H](CCCC*)*")
    if not mol.HasSubstructMatch(smart_pattern):
        return False, "Does not have correct L-lysine stereochemistry"
        
    # Check for modifications at amino groups or carboxy group
    amino_mod = False
    carboxy_mod = False
    
    # Check terminal amino group modifications
    term_nh2_pattern = Chem.MolFromSmarts("CCCCN")
    if mol.HasSubstructMatch(term_nh2_pattern):
        matches = mol.GetSubstructMatches(term_nh2_pattern)
        for match in matches:
            n_atom = mol.GetAtomWithIdx(match[-1])
            if len(n_atom.GetNeighbors()) > 2:  # Modified if more than 2 neighbors
                amino_mod = True
                
    # Check alpha amino group modifications  
    alpha_nh2_pattern = Chem.MolFromSmarts("[NH2][C@@H]")
    if not mol.HasSubstructMatch(alpha_nh2_pattern):
        amino_mod = True
        
    # Check carboxy modifications
    carboxy_pattern = Chem.MolFromSmarts("C(=O)O")
    if not mol.HasSubstructMatch(carboxy_pattern):
        carboxy_mod = True
        
    if amino_mod and carboxy_mod:
        return True, "L-lysine derivative with both amino and carboxy modifications"
    elif amino_mod:
        return True, "L-lysine derivative with amino group modification"
    elif carboxy_mod:
        return True, "L-lysine derivative with carboxy group modification"
    else:
        return False, "No modifications found on L-lysine core structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25095',
                          'name': 'L-lysine derivative',
                          'definition': 'A proteinogenic amino acid derivative '
                                        'resulting from reaction of L-lysine '
                                        'at the amino group or the carboxy '
                                        'group, or from the replacement of any '
                                        'hydrogen of L-lysine by a heteroatom.',
                          'parents': ['CHEBI:53079', 'CHEBI:83811']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 112565,
    'num_false_negatives': 7,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9990503408122693}


"""
Classifies: CHEBI:26512 quinolinemonocarboxylic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_quinolinemonocarboxylic_acid(smiles: str):
    """
    Determines if a molecule is a quinoline monocarboxylic acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a quinoline monocarboxylic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for quinoline core structure
    quinoline_pattern = Chem.MolFromSmarts('c1cccc2c1ncc[c]2')
    if not mol.HasSubstructMatch(quinoline_pattern):
        return False, "No quinoline core structure found"
        
    # Count carboxylic acid groups
    carboxyl_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    carboxyl_matches = mol.GetSubstructMatches(carboxyl_pattern)
    
    if len(carboxyl_matches) == 0:
        return False, "No carboxylic acid group found"
    elif len(carboxyl_matches) > 1:
        return False, "More than one carboxylic acid group found"
        
    # Verify that carboxylic acid is attached to quinoline
    quinoline_matches = mol.GetSubstructMatches(quinoline_pattern)
    quinoline_atoms = set()
    for match in quinoline_matches:
        quinoline_atoms.update(match)
        
    carboxyl_carbon = carboxyl_matches[0][0]
    
    # Check if carboxyl group is connected to quinoline system
    carboxyl_neighbors = [x.GetIdx() for x in mol.GetAtomWithIdx(carboxyl_carbon).GetNeighbors()]
    is_connected = any(neighbor in quinoline_atoms for neighbor in carboxyl_neighbors)
    
    if not is_connected:
        return False, "Carboxylic acid group not connected to quinoline system"
        
    return True, "Contains quinoline core with one attached carboxylic acid group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26512',
                          'name': 'quinolinemonocarboxylic acid',
                          'definition': 'Any aromatic carboxylic acid that '
                                        'contains a quinoline moiety that is '
                                        'substituted by one carboxy '
                                        'substituent.',
                          'parents': ['CHEBI:26513', 'CHEBI:33859']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 94,
    'num_true_negatives': 183808,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.030927835051546393,
    'recall': 1.0,
    'f1': 0.06000000000000001,
    'accuracy': 0.999488866534352}


"""
Classifies: CHEBI:18282 nucleobase
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_nucleobase(smiles: str):
    """
    Determines if a molecule is a nucleobase.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a nucleobase, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Define the nucleobase SMARTS patterns
    purine_pattern = "[r5]1([r5,r6])c([r5,r6])nc([r5,r6])c1[r5,r6]"
    pyrimidine_pattern = "[r5,r6]c1[r5,r6]c([r5,r6])c(=O)[r5,r6]c(=O)[r5,r6]1"
    cytosine_pattern = "Nc1cc[nH]c(=O)n1"
    thymine_pattern = "Cc1c[nH]c(=O)[nH]c1=O"

    purine_query = Chem.MolFromSmarts(purine_pattern)
    pyrimidine_query = Chem.MolFromSmarts(pyrimidine_pattern)
    cytosine_query = Chem.MolFromSmarts(cytosine_pattern)
    thymine_query = Chem.MolFromSmarts(thymine_pattern)

    # Check if the molecule matches the nucleobase SMARTS patterns
    is_purine = mol.HasSubstructMatch(purine_query)
    is_pyrimidine = mol.HasSubstructMatch(pyrimidine_query)
    is_cytosine = mol.HasSubstructMatch(cytosine_query)
    is_thymine = mol.HasSubstructMatch(thymine_query)

    if is_purine:
        return True, "Molecule is a purine nucleobase"
    elif is_pyrimidine:
        return True, "Molecule is a pyrimidine nucleobase"
    elif is_cytosine:
        return True, "Molecule is cytosine"
    elif is_thymine:
        return True, "Molecule is thymine"
    else:
        return False, "Molecule is not a nucleobase"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18282',
                          'name': 'nucleobase',
                          'definition': 'That part of DNA or RNA that may be '
                                        'involved in pairing.',
                          'parents': ['CHEBI:38101']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('O=C1C(=O)C2=C(NC3=C2C=CC=C3)C(=C1C)CC(C)C', 'Molecule is a "
               "purine nucleobase'), "
               "('COc1ccc2c(c1)[nH]c1c(CC=C(C)C)c(O)c(C=O)cc21', 'Molecule is "
               "a purine nucleobase'), "
               "('C1=CC=C(C=C1)C2=C(N3C(=O)C(=CC4=CC(=CC=C4)O)SC3=N2)C5=CC=CC=C5', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(C1=NC2=C(C(=C(CCCCC(C)C)C3=C2C4=C(C=CC=C4)N3)C)O[C@H]1C=5C6=C(C=CC=C6)NC5)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C2=C3C(C=4C[C@H]5[C@](C4N3[C@H]1C(=C)C)([C@@]6([C@H]([C@](/C=C/C=C(/C(=O)O)\\\\C)([C@@H](O)CC6)C)CC5)C)C)=CC7=C2[C@@H](O)[C@@H]8C(OC(C=C78)(C)C)(C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O1C2C3N4C(CC(C3C1)C(C4)=CC)C=5NC6=C(C25)C=CC(OC)=C6', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C=C2C(=CC[C@]3([C@@]2(O)CC[C@@H]4[C@@]3(C=5NC=6C=C7C8=CC(OC([C@H]8CC7=CC6C5C4)(C)C)(C)C)C)C)O[C@@H]1C(O)(C)C', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)CC5CCCC5)CC6=NC=CS6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)C(=O)C5=CC(=CC=C5)F)CC6CC6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C(=O)C2=C(NC3=C2C=CC=C3)C(=C1C)CCCCCCC(C)C', 'Molecule "
               "is a purine nucleobase'), "
               "('COc1ccc2c(c1)[nH]c1c(O)cc(C=O)cc21', 'Molecule is a purine "
               "nucleobase'), "
               "('O=C1N2[C@]3(CC4[C@@]1(NC3=O)C(=O)C5=C(N(O)C6=C5C=CC=7OC(C=CC67)(C)C)C4(C)C)CCC2', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(O)C(=O)C1=C2NC=3C=CC=CC3C2=CC4=C1C5=C(C=CC=C5)N4', "
               "'Molecule is a purine nucleobase'), "
               "('CC(O)C1CCc2c3C=CC(=O)c4c(O)c5C(C)OC(CC(O)=O)Cc5c(n12)c34', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C=C2[C@@]3(CC[C@]4([C@H]2CC[C@@H]5[C@@]4(C=6NC=7C=CC(=CC7C6C5)C(C=C)(C)C)C)C)O[C@@H]1C(O3)(C)C', "
               "'Molecule is a purine nucleobase'), "
               "('C=Cc1cncc2c1cc1-c3[nH]c4ccccc4c3CCn1c2=O', 'Molecule is a "
               "purine nucleobase'), ('CC1(C)[C@@H](O)C(=O)c2c1[nH]c1ccccc21', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1[C@]23C=4N(C=5C2=CC=CC5)C(=O)C=CC4[C@@](C1)(/C(/CN(CC3)C)=C\\\\CO)[H]', "
               "'Molecule is a purine nucleobase'), "
               "('CCC(=O)N1CC2(CCN(CC2)CC3=CC=CC=C3Cl)C4=C([C@H]1CO)NC5=C4C=CC(=C5)OC', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(O[C@@H]1[C@]([C@H]2[C@]([C@]3(C=4NC=5C=CC=CC5C4C[C@@H]3C[C@H]2O)C)(C)CC1)(CCC=C(C)C)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N[C@]23[C@H](C(C=4NC5=C(C4C2)C=CC6=C5C(=O)CC(O6)(C)C)(C)C)C[C@]17N(CC[C@@H]7C)C3', "
               "'Molecule is a purine nucleobase'), "
               "('COC1=CC2=C(C=C1)C3=C(N2)[C@@H](N(CC34CCN(CC4)CC5=CC(=CC=C5)F)C(=O)C6CCOCC6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('S=C=N[C@H]1[C@](C=C)(CC[C@H]2[C@H]1C=3C4=C(C=CC=C4)NC3C2(C)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N2[C@@]3(C(=O)NC14C(C(C5=[N+]([O-])C6=C(C75[C@@H]4N8C(=O)[C@@]9%10N(CCC9)C(C8%11C(C(C)(C)C%12=C([C@H]7%11)C%13=C(C%14=C(OC(C)(C)C=C%14)C=C%13)N%12)C%10)=O)C=CC%15=C6C=CC(O%15)(C)C)(C)C)C3)CCC2', "
               "'Molecule is a purine nucleobase'), "
               "('COC1=CC2=C(C=C1)C3=C(N2)[C@@H](N(CC34CCN(CC4)CC5=CN=CC=C5)CC6=NC=CS6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('COc1cc(C=O)cc2c3ccccc3[nH]c12', 'Molecule is a purine "
               "nucleobase'), "
               "('CCOC1=CC2=C(C=C1)NC3=C2N=CN=C3NCCC4=CC(=C(C=C4)OC)OC', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=CC2=C(N(C)C[C@]23C=4NC=5C=CC=CC5C4C=6C3=C(NC6)C(=O)OC)C=C1', "
               "'Molecule is a purine nucleobase'), "
               "('OC=1C=C2NC=3C(C2=CC1C(OC)=O)=CC=CC3', 'Molecule is a purine "
               "nucleobase'), ('O(C1=C(O)C2=C(NC3=C2C=C(OC)C=C3)C(=C1C)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=C2N(C=3C=4NC=5C(Cl)=CC=CC5C4C6=C(C3C2=CC(=C1)OC)C(=O)NC6=O)[C@@H]7O[C@@H]([C@@H](OC)[C@@H]([C@H]7O)O)CO', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=CC2=C(NC3=C2C=C(C(=O)O[C@H]4[C@@H](N)[C@H](O[C@H](C4)O[C@H]5[C@@](O)(C[C@H](O[C@@H]6[C@@H](O[C@H](OC7C8C=CC9CC(O)C(CC%10(C=C(C(C%11(C(C(=C(C9(C8CCC7)C)O)C(=O)N%11)=O)C%10)C)C(=O)O)C)CC)CC6)C)O[C@@H]5C)C)C)N3)C=C1', "
               "'Molecule is a purine nucleobase'), "
               "('Cl[C@H]1[C@](C(C#N)=C2C=3C4=C(C=CC=C4)NC3C([C@H]2C1)(C)C)(C=C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1NCC=2C1=C3C=4N([C@@H]5OC(C)=C([C@@](N6C4C2C7=C6C=CC=C7)(OC)C5)OC)C8=C3C=CC=C8', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=C2C3=C4C=5[C@@](OC([C@@]6([C@@]3([C@](C6)(C(C2)=C)[H])[H])[H])(C)C)([C@]7([C@@]([C@@]8([C@](O)([C@@]9%10O[C@@]9([C@@H](O)[C@H](O[C@]%10(CC8)[H])C(C)=C)[H])CC7)C)(C5NC4=C1)C)[H])[H]', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C=2NC=3C=CC=CC3C2C(=C1C4=C(NC5=C4C=CC=C5)/C(/C(=O)OC)=C\\\\C6=CC=C(O)C=C6)C(=O)OC', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)C(=O)CC5=CC=NC=C5)CC6=CC=NC=C6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('O[C@]12[C@@]([C@@]3([C@@H](CC1)CC4=C3NC5=C4C=C6C(=C5)C=7[C@H](C6)C(OC(C7)(C)C)(C)C)C)(CC[C@@H]8O[C@@H]([C@H](OC(=O)C)C=C28)C(O)(C)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('CC[C@@]1(CN2CCC=3C4=CC=CC=C4NC3[C@@]2(C[C@@]1(CC=O)[H])[H])[H]', "
               "'Molecule is a purine nucleobase'), "
               "('CC1=CC=C(C=C1)S(=O)(=O)N2CCC3(CC2)CN[C@@H](C4=C3C5=C(N4C)C=C(C=C5)OC)CO', "
               "'Molecule is a purine nucleobase'), "
               "('CC(=O)c1nc(C(N)=O)c(O)c2c3ccccc3[nH]c12', 'Molecule is a "
               "purine nucleobase'), ('CC1(C)C(=O)C(=O)c2c1[nH]c1cc(O)ccc21', "
               "'Molecule is a purine nucleobase'), "
               "('CCC(=O)N1CCC2(CC1)CN([C@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)S(=O)(=O)C5=CC=CC=C5', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1NC(=O)C2=C1C3=C(N([C@@H]4O[C@@H]([C@@H](O)[C@@H]([C@H]4O)O)CO)C5=C3C=CC=C5)C6=C2C7=C(C=CC=C7)N6', "
               "'Molecule is a purine nucleobase'), "
               "('CC1=C2C3=C(C=CC(=C3)[N+](=O)[O-])NC2=C(C=C1)C', 'Molecule is "
               "a purine nucleobase'), "
               "('C=C[C@]1(CN2CCC=3C=4C=CC(=CC4NC3[C@@]2(C[C@@]1(C[C@@]5(C6=C(CCN5C)C7=CC=CC=C7N6)[H])[H])[H])O)[H]', "
               "'Molecule is a purine nucleobase'), "
               "('COc1ccc(cc1O)-c1c2c3cc(OC)c(OS(O)(=O)=O)cc3oc(=O)c2n2ccc3cc(OC)c(OC)cc3c12', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N2[C@H](C(=O)N3[C@]1(O)[C@@H](OC)C=4C5=C(C=C(OC)C=C5)NC4[C@@H]3CC(O)(C)C)CCC2', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)C(=O)NC5CCCC5)CC6CC6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('CN(C)CC(=O)N1CCC2(CC1)CN([C@@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)C(=O)C5=CN=CC=C5', "
               "'Molecule is a purine nucleobase'), "
               "('N1=C(C=2NC=3C=CC=CC3C2CC1)C=4C5=C(C=CC=C5)NC4', 'Molecule is "
               "a purine nucleobase'), "
               "('OCc1ccc(o1)-c1nc(cc2c3ccccc3[nH]c12)C(O)=O', 'Molecule is a "
               "purine nucleobase'), "
               "('COC1=CC2=C(C=C1)NC3=C2NC=[N+](C3=O)CCCN4CCOCC4', 'Molecule "
               "is a purine nucleobase'), ('C[C@H]1NCCc2c1[nH]c1ccccc21', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N(CC=2C1=C3C=4N([C@@H]5O[C@](N6C4C2C7=C6C=CC=C7)([C@H](OC)\\\\C(\\\\C5)=N\\\\O)C)C8=C3C=CC=C8)COC', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)C(=O)COC)CC5=C(C=CC(=C5)F)F)CO', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=C2C3=C4C=5[C@@H](OC([C@@H]6[C@]3(O)[C@H](C6)C(C2)=C)(C)C)O[C@@]7([C@](C5NC4=C1)([C@@]8(C([C@@]9%10O[C@@H]9[C@@H](O)[C@H](OC%10CC8)C(=C)C)CC7)C)C)O', "
               "'Molecule is a purine nucleobase'), "
               "('[H][C@]12CC[C@H](O)[C@H](C(=O)OC)[C@@]1([H])C[C@]1([H])N(CCc3c1[nH]c1ccccc31)C2', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N2[C@@]3(C(=O)N[C@]14C(=O)C=5C6=C(C7=C(OC(C)(C)C=C7)C=C6)N(C5C([C@@H]4C3)(C)C)O)CCC2', "
               "'Molecule is a purine nucleobase'), "
               "('O(C1=C(O)C2=C(NC3=C2C=C(CC=C(C)C)C=C3)C(=C1C)C[C@H](O)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(OCC)C[C@@H]1OCCC2=C1NC=3C(=CC=CC23)CC', 'Molecule is a "
               "purine nucleobase'), ('S1C=2N(OC)C=3C(C2CN=C1S(=O)C)=CC=CC3', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(O)[C@]1([C@H]2[C@@](OC=3C=C4C5=C(C=CC=C5)NC4=CC3CC2)(CC[C@@H]1O)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C2=C(C3=C(NC4=C3C[C@@H]5CC[C@@]6([C@@]([C@@]45C)(CC[C@H]7[C@@]68O[C@@H]8[C@@H]9O[C@H](C=C(C)C)OC([C@H]9O7)(C)C)C)O)C=C2)C[C@@H]%10[C@H]1C(OC%10(C)C)(C)C', "
               "'Molecule is a purine nucleobase'), "
               "('CC1(C)[C@@H](O)C(=O)c2c1[nH]c1cc(O)ccc21', 'Molecule is a "
               "purine nucleobase'), "
               "('O=C(O)C1=NC(=C2NC3=C(C2=C1)C=CC=C3)C(=O)CCCC(C)C', 'Molecule "
               "is a purine nucleobase'), "
               "('BrC1=C2C3=C4C=5[C@@H](OC([C@@H]6[C@]3(O)[C@H](C6)C(C2)=C)(C)C)[C@@H]7CC[C@@]8([C@@]([C@]7(C5NC4=C1)C)(CC[C@H]9[C@@]8%10O[C@@H]%10[C@@H](O)[C@H](O9)C(=C)C)C)O', "
               "'Molecule is a purine nucleobase'), "
               "('C=C[C@H]1CN2CCc3c([nH]c4ccccc34)[C@@H]2C[C@@H]1CC=O', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=C2C3=C4C=5[C@@H](OC([C@@H]6[C@]3(O)[C@H](C6)C(C2)=C)(C)C)[C@]7(O)CC[C@@]8([C@@]([C@]7(C5NC4=C1)C)(CC[C@H]9[C@@]8%10O[C@@H]%10[C@@H](O)[C@H](O9)C(=C)C)C)O', "
               "'Molecule is a purine nucleobase'), "
               "('ClC1=CC=2C=3C=4C(=CNC4)[C@@]5(C=6C=C(Cl)C=CC6N([C@@H]5C3NC2C=C1)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=c1n2CCc3cccc(c23)c2[nH]c3ccccc3c12', 'Molecule is a purine "
               "nucleobase'), ('COc1ccc2[nH]c3c(Cc4ccc(O)cc4)nccc3c2c1', "
               "'Molecule is a purine nucleobase'), "
               "('COC1=CC2=C(C=C1)C3=C(N2)[C@H](N(CC34CCN(CC4)CC5=CN=CC=C5)C(=O)C6=CC=C(C=C6)F)CO', "
               "'Molecule is a purine nucleobase'), "
               "('CC1(C2=C(C[C@@H]3N1C(=O)N(C3=O)CC4=CC(=CC=C4)F)C5=CC=CC=C5N2)C', "
               "'Molecule is a purine nucleobase'), "
               "('CCCNC(=O)N1CCC2(CC1)CN([C@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)C(=O)COC', "
               "'Molecule is a purine nucleobase'), "
               "('CC1=NN2C(=C3C(=C2C(=C1)C4=CC=CC=C4)N(C(=O)N(C3=O)C)C)C5=CC=CC=C5', "
               "'Molecule is a purine nucleobase'), "
               "('O(CC(CNC(C)C)O)C=1C=2C=3C(NC2C=CC1)=CC=CC3', 'Molecule is a "
               "purine nucleobase'), "
               "('BrC1=C2N(C=3C=4NC=5C(Br)=CC=CC5C4C6=C(C3C2=CC=C1)C(=O)NC6=O)C7OC(C(OC)C(C7O)O)CO', "
               "'Molecule is a purine nucleobase'), "
               "('O=C(OCC1=NC(=C2NC3=C(C2=C1)C=CC=C3)[C@@H](O)COC(=O)C)C', "
               "'Molecule is a purine nucleobase'), "
               "('O1[C@H](C(O)(C)C)[C@@H](O)[C@@H]2[C@]3([C@@H]1CC[C@]4([C@@]3(O)CC[C@@H]5[C@@]4(C=6NC=7C=CC=C(C7C6C5)CC=C(C)C)C)C)O2', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1N2C=3[C@@]4([C@]5(N(CC4)C[C@H]([C@](C5)(C3C=C1C(OCC)=O)[H])CC)[H])C=6C2=CC=CC6', "
               "'Molecule is a purine nucleobase'), "
               "('O[C@]12[C@@]([C@@]3([C@@](CC1)(CC4=C3NC5=C4C=C(C(=C5)CC=C(C)C)CC=C(C)C)[H])C)(CC[C@@]6(O[C@@](C(=O)C=C26)(C(O)(C)C)[H])[H])C', "
               "'Molecule is a purine nucleobase'), "
               "('O(C1=C(OC)C2=C(NC3=C2C=CC=C3)C(=C1C)C)C', 'Molecule is a "
               "purine nucleobase'), "
               "('O1[C@@]2(N3C4=C5N([C@]1(C[C@@H](N)[C@H]2O)[H])C=6C(C5=C7C(=C4C=8C3=CC=CC8)CNC7=O)=CC=CC6)C', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1[C@@](O)([C@](O)(C=2NC3=C(C2C1)C=CC=C3)C)C', 'Molecule "
               "is a purine nucleobase'), "
               "('COC1=CC2=C(C=C1)C3=C(N2)[C@@H](N(CC34CCN(CC4)CC5CCCC5)C(=O)CN6CCOCC6)CO', "
               "'Molecule is a purine nucleobase'), "
               "('O1C(C=2NC=3C=CC=CC3C2CC1)(C)C', 'Molecule is a purine "
               "nucleobase'), "
               "('COC(=O)CC(O)N(C)C(=O)c1cc2c3ccccc3[nH]c2c(n1)C(C)=O', "
               "'Molecule is a purine nucleobase'), "
               "('CCC(=O)N1CC2(CCN(CC2)CC3=CC=CC=C3Cl)C4=C([C@@H]1CO)NC5=C4C=CC(=C5)OC', "
               "'Molecule is a purine nucleobase'), "
               "('Brc1ccc2[nH]c3cnccc3c2c1', 'Molecule is a purine "
               "nucleobase'), "
               "('COc1c2-c3[nH]c4ccc(Cl)cc4c3[C@@]3(O)C(=O)N(C)C(=O)[C@@]3(O)n2c2ccc(Cl)cc12', "
               "'Molecule is a purine nucleobase'), "
               "('CC[C@H]1C[C@@H]2CN3CCc4c([nH]c5c([C@H]6CC[C@H]7NCCc8c7n6c6ccccc86)c(O)ccc45)[C@](C2)([C@H]13)C(=O)OC', "
               "'Molecule is a purine nucleobase'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)C(=O)C5=CC=CC=C5F)C(=O)NC6=CC=C(C=C6)F)CO', "
               "'Molecule is a purine nucleobase'), "
               "('CCN1CCN2C3=C(CCCC31)C4=C2C(=CC=C4)C', 'Molecule is a purine "
               "nucleobase'), "
               "('C1CCC(CC1)C2C3=C(CCN2CC4=NC=CN4)C5=CC=CC=C5N3', 'Molecule is "
               "a purine nucleobase'), "
               "('O1C(OC2=CC=3C=4CCN(C4NC3C=C2)C(=O)C)C(O)C(O)C(O)C1C(O)=O', "
               "'Molecule is a purine nucleobase'), "
               "('[C@H]12N3C=4N=C(NC(C4N=C3[C@H]([C@]([C@H](C1)O)(O2)[H])OP(O)(O)=O)=O)N', "
               "'Molecule is a purine nucleobase'), "
               "('[C@H]12N3C=4N=CN=C(C4N=C3[C@H]([C@]([C@H](C1)O)(O2)[H])O)N', "
               "'Molecule is a purine nucleobase'), "
               "('[H][C@]12C[C@@H](OC(=O)c3cc(OC)c(OC)c(OC)c3)[C@H](OC)[C@@H](C(=O)OC)[C@@]1([H])C[C@@]1([H])N(CCc3c1[nH]c1cc(OC)ccc31)C2', "
               "'Molecule is a purine nucleobase'), "
               "('O=C1C(=O)C2=C(NC3=C2C=C(CC4=C(CC(C)(C)CC4)C)C=C3)C(=C1C)C[C@H](O)C', "
               "'Molecule is a purine nucleobase')]\n"
               "False negatives: [('Nc1cc[nH]c(=O)n1', 'Molecule is not a "
               "nucleobase')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 12668,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9921685331662621}


"""
Classifies: CHEBI:25388 monohydroxybenzoate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_monohydroxybenzoate(smiles: str):
    """
    Determines if a molecule is a monohydroxybenzoate (benzoate with single hydroxy substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a monohydroxybenzoate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for aromatic ring
    rings = mol.GetRingInfo()
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)

    if not aromatic_rings:
        return False, "No aromatic 6-membered ring found"

    # Check for benzoate group (carboxylate attached to benzene ring)
    carboxylate_pattern = Chem.MolFromSmarts('c-C([O-])=O')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No benzoate group found"

    # Count hydroxy groups attached to the aromatic ring
    hydroxy_pattern = Chem.MolFromSmarts('c-[OH]')
    hydroxy_matches = mol.GetSubstructMatches(hydroxy_pattern)
    
    if len(hydroxy_matches) == 0:
        return False, "No hydroxy substituent found"
    elif len(hydroxy_matches) > 1:
        return False, f"Found {len(hydroxy_matches)} hydroxy groups, expected exactly one"
    
    # Verify hydroxy is attached to the same aromatic ring as carboxylate
    ring_atoms = set(aromatic_rings[0])
    hydroxy_carbon = mol.GetAtomWithIdx(hydroxy_matches[0][0])
    if hydroxy_carbon.GetIdx() not in ring_atoms:
        return False, "Hydroxy group not attached to main aromatic ring"
        
    # Get position of hydroxy group relative to carboxylate
    carboxylate_matches = mol.GetSubstructMatches(carboxylate_pattern)
    carboxylate_carbon = mol.GetAtomWithIdx(carboxylate_matches[0][0])
    
    positions = []
    for i, atom_idx in enumerate(aromatic_rings[0]):
        if atom_idx == hydroxy_carbon.GetIdx():
            hydroxy_pos = i + 1
            positions.append(f"{hydroxy_pos}-hydroxy")
            
    return True, f"Monohydroxybenzoate: {', '.join(positions)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25388',
                          'name': 'monohydroxybenzoate',
                          'definition': 'A hydroxybenzoate carrying a single '
                                        'hydroxy substituent at unspecified '
                                        'position.',
                          'parents': ['CHEBI:24675']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 60,
    'num_true_negatives': 183855,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.03225806451612903,
    'recall': 1.0,
    'f1': 0.0625,
    'accuracy': 0.9996737658835235}


"""
Classifies: CHEBI:25271 methyl-branched fatty acyl-CoA
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_methyl_branched_fatty_acyl_CoA(smiles: str):
    """
    Determines if a molecule is a methyl-branched fatty acyl-CoA.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a methyl-branched fatty acyl-CoA, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for CoA substructure
    coA_pattern = Chem.MolFromSmarts('[#16]-[#6]-[#6]-[#7]-[#6](=[#8])-[#6]-[#6]-[#7]-[#6](=[#8])-[#6]-[#6](O)-[#6](C)(C)-[#6]-O-P(=[#8])(O)-O-P(=[#8])(O)-O-[#6]-[#6]-1-O-[#6](-[#6](-[#6]-1-O-P(=[#8])(O)O)O)n1cnc2c(N)ncnc12')
    if not mol.HasSubstructMatch(coA_pattern):
        return False, "Missing CoA moiety"
    
    # Check for thioester linkage
    thioester_pattern = Chem.MolFromSmarts('[#6](=O)-[#16]')
    if not mol.HasSubstructMatch(thioester_pattern):
        return False, "Missing thioester linkage"
        
    # Check for methyl branching on carbon chain
    methyl_branch_pattern = Chem.MolFromSmarts('[#6]-[#6](-[#6])-[#6]')
    if not mol.HasSubstructMatch(methyl_branch_pattern):
        return False, "No methyl branching found"
        
    # Count number of carbons in main chain
    carbon_chain_pattern = Chem.MolFromSmarts('[#6](-[#16])-[#6]-[#6]')
    matches = mol.GetSubstructMatches(carbon_chain_pattern)
    if not matches:
        return False, "No carbon chain found"
        
    # Find number of methyl branches
    branch_matches = len(mol.GetSubstructMatches(methyl_branch_pattern))
    
    return True, f"Methyl-branched fatty acyl-CoA with {branch_matches} methyl branch(es)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25271',
                          'name': 'methyl-branched fatty acyl-CoA',
                          'definition': 'A branched-chain fatty acyl-CoA that '
                                        'results from the formal condensation '
                                        'of the thiol group of coenzyme A with '
                                        'the carboxy group of any '
                                        'methyl-branched fatty acid.',
                          'parents': ['CHEBI:61912']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183877,
    'num_false_negatives': 5,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999728086490249}


"""
Classifies: CHEBI:25275 methylaniline
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_methylaniline(smiles: str):
    """
    Determines if a molecule is a methylaniline (a substituted aniline carrying one or more methyl groups at unspecified positions).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a methylaniline, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains an aniline substructure
    aniline_pattern = Chem.MolFromSmarts('c1ccc(N)cc1')
    if not mol.HasSubstructMatch(aniline_pattern):
        return False, "Molecule does not contain an aniline substructure"

    # Check if the molecule contains at least one methyl group
    methyl_pattern = Chem.MolFromSmarts('C')
    if not any(mol.HasSubstructMatch(methyl_pattern, useChirality=False)):
        return False, "Molecule does not contain any methyl groups"

    # Check if the methyl groups are attached to the aniline substructure
    aniline_atoms = list(mol.GetSubstructMatch(aniline_pattern))
    methyl_groups = []
    for atom_idx in aniline_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C' and sum(1 for bond in neighbor.GetBonds() if bond.GetBondType() == Chem.BondType.SINGLE) == 3:
                methyl_groups.append(neighbor.GetIdx())

    if not methyl_groups:
        return False, "Methyl groups are not attached to the aniline substructure"

    return True, f"Methylaniline with {len(set(methyl_groups))} methyl group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25275',
                          'name': 'methylaniline',
                          'definition': 'A substituted aniline carrying one or '
                                        'more methyl groups at unspecified '
                                        'positions.',
                          'parents': ['CHEBI:48975']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'bool' object is not iterable",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:172368 brominated flame retardant
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_brominated_flame_retardant(smiles: str):
    """
    Determines if a molecule is a brominated flame retardant.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a brominated flame retardant, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains bromine
    if 'Br' not in Chem.MolToSmiles(mol):
        return False, "Molecule does not contain bromine"

    # Check for presence of aromatic rings
    rings = mol.GetRingInfo().AtomRings()
    aromatic_rings = [ring for ring in rings if all(mol.GetAtomWithIdx(idx).GetIsAromatic() for idx in ring)]

    # Check if the molecule has an aromatic ring and a bromine substituent
    has_bromine_substituent = False
    for ring in aromatic_rings:
        for idx in ring:
            atom = mol.GetAtomWithIdx(idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'Br' and neighbor.GetIdx() not in ring:
                    has_bromine_substituent = True
                    break

    if has_bromine_substituent:
        return True, "Molecule is a brominated flame retardant"
    else:
        return False, "Molecule does not meet the criteria for a brominated flame retardant"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:172368',
                          'name': 'brominated flame retardant',
                          'definition': 'Any organobromine compound that is '
                                        'used as a flame retardant. These '
                                        'chemicals are widely incorporated as '
                                        'additives in consumer products such '
                                        'as electronics, vehicles, '
                                        'polyurethane foams etc, to make them '
                                        'less flammable.',
                          'parents': ['CHEBI:37141']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 13089,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9924184988627748}


"""
Classifies: CHEBI:27177 L-tyrosine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem import rdmolops

def is_L_tyrosine_derivative(smiles: str):
    """
    Determines if a molecule is an L-tyrosine derivative.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an L-tyrosine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains the L-tyrosine scaffold
    tyrosine_scaffold = Chem.MolFromSmiles('N[C@@H](Cc1ccc(O)cc1)C(O)=O')
    if mol.GetSubstructMatch(tyrosine_scaffold) == []:
        return False, "Molecule does not contain the L-tyrosine scaffold"

    # Check for modifications at the amino group, carboxy group, or hydrogen replacement
    modified = False
    reason = []

    # Check for modifications at the amino group
    amino_group = Chem.MolFromSmiles('N')
    amino_group_match = mol.GetSubstructMatches(amino_group)
    if amino_group_match:
        amino_group_atom = mol.GetAtomWithIdx(amino_group_match[0][0])
        if not list(amino_group_atom.GetNeighbors())[0].GetSymbol() == 'H':
            modified = True
            reason.append("Modified at amino group")

    # Check for modifications at the carboxy group
    carboxy_group = Chem.MolFromSmiles('C(O)=O')
    carboxy_group_match = mol.GetSubstructMatches(carboxy_group)
    if carboxy_group_match:
        carboxy_group_atom = mol.GetAtomWithIdx(carboxy_group_match[0][0])
        neighbors = list(carboxy_group_atom.GetNeighbors())
        if any(neighbor.GetSymbol() != 'O' and neighbor.GetSymbol() != 'H' for neighbor in neighbors):
            modified = True
            reason.append("Modified at carboxy group")

    # Check for hydrogen replacement by heteroatoms
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetTotalNumHs() < atom.GetImplicitValence():
            modified = True
            reason.append("Hydrogen replaced by heteroatom")
            break

    if modified:
        return True, ", ".join(reason)
    else:
        return False, "Unmodified L-tyrosine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27177',
                          'name': 'L-tyrosine derivative',
                          'definition': 'A proteinogenic amino acid derivative '
                                        'resulting from reaction of L-tyrosine '
                                        'at the amino group or the carboxy '
                                        'group, or from the replacement of any '
                                        'hydrogen of L-tyrosine by a '
                                        'heteroatom.',
                          'parents': [   'CHEBI:62761',
                                         'CHEBI:83811',
                                         'CHEBI:84144']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 21,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.20634920634920634}


"""
Classifies: CHEBI:18244 1-alkyl-sn-glycero-3-phosphoethanolamine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_1_alkyl_sn_glycero_3_phosphoethanolamine(smiles: str):
    """
    Determines if a molecule is a 1-alkyl-sn-glycero-3-phosphoethanolamine.
    A glycerophosphoethanolamine that is sn-glycero-3-phosphoethanolamine carrying any alkyl substituent at position 1.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 1-alkyl-sn-glycero-3-phosphoethanolamine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a glycerol backbone
    glycerol_smarts = "[C@](CO)(CO)CO"
    glycerol_match = mol.GetSubstructMatches(Chem.MolFromSmarts(glycerol_smarts))
    if not glycerol_match:
        return False, "No glycerol backbone found"

    # Check for the presence of a phosphate group
    phosphate_smarts = "OP(O)(=O)O"
    phosphate_match = mol.GetSubstructMatches(Chem.MolFromSmarts(phosphate_smarts))
    if not phosphate_match:
        return False, "No phosphate group found"

    # Check for the presence of an ethanolamine group
    ethanolamine_smarts = "OCCN"
    ethanolamine_match = mol.GetSubstructMatches(Chem.MolFromSmarts(ethanolamine_smarts))
    if not ethanolamine_match:
        return False, "No ethanolamine group found"

    # Check if the phosphate group is connected to the glycerol backbone and the ethanolamine group
    phosphate_atom = mol.GetAtomWithIdx(list(phosphate_match[0])[0])
    phosphate_neighbors = [mol.GetAtomWithIdx(nbr_idx) for nbr_idx in phosphate_atom.GetNeighbors()]

    glycerol_atom = None
    ethanolamine_atom = None
    for neighbor in phosphate_neighbors:
        if neighbor.GetSmarts() == "CO":
            glycerol_atom = neighbor
        elif neighbor.GetSmarts() == "OCCN":
            ethanolamine_atom = neighbor

    if not glycerol_atom or not ethanolamine_atom:
        return False, "Phosphate group not connected to glycerol backbone and ethanolamine group"

    # Check for the presence of an alkyl substituent at position 1
    alkyl_substituent = False
    for neighbor in glycerol_atom.GetNeighbors():
        if neighbor.GetSymbol() == "C" and neighbor.GetAtomicNum() != 8:
            alkyl_substituent = True
            break

    if alkyl_substituent:
        return True, "1-alkyl-sn-glycero-3-phosphoethanolamine"
    else:
        return False, "No alkyl substituent at position 1"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18244',
                          'name': '1-alkyl-sn-glycero-3-phosphoethanolamine',
                          'definition': 'A glycerophosphoethanolamine that is '
                                        'sn-glycero-3-phosphoethanolamine '
                                        'carrying any alkyl substituent at '
                                        'position 1.',
                          'parents': ['CHEBI:138320']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetAtomWithIdx(Mol, Atom)\n'
             'did not match C++ signature:\n'
             '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int idx)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:140322 triketone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_triketone(smiles: str):
    """
    Determines if a molecule contains exactly three ketone groups (C=O).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains exactly 3 ketone groups, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find all ketone groups (C=O)
    # This pattern matches a carbon with a double bond to oxygen, excluding carboxylic acids, esters, and amides
    ketone_pattern = Chem.MolFromSmarts('[C;!$(C(=O)O);!$(C(=O)OC);!$(C(=O)N)](=O)')
    matches = mol.GetSubstructMatches(ketone_pattern)
    
    # Count number of ketone groups
    num_ketones = len(matches)
    
    if num_ketones == 3:
        # Get the atoms involved in each ketone group for detailed reporting
        ketone_details = []
        for match in matches:
            carbon = mol.GetAtomWithIdx(match[0])
            neighbors = [n.GetSymbol() for n in carbon.GetNeighbors() if n.GetAtomicNum() != 8]  # exclude oxygen
            ketone_details.append(f"C(=O) bonded to {','.join(neighbors)}")
            
        return True, f"Contains exactly 3 ketone groups: {'; '.join(ketone_details)}"
    elif num_ketones > 3:
        return False, f"Contains {num_ketones} ketone groups (more than 3)"
    else:
        return False, f"Contains only {num_ketones} ketone groups (less than 3)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140322',
                          'name': 'triketone',
                          'definition': 'A compound that contains three ketone '
                                        'functionalities.',
                          'parents': ['CHEBI:17087']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('[C@@]12([C@]3([C@](C[C@@]1([C@]4([H])CC[C@H]([C@]4(C3)[H])C)C=O)(C=C2C(C)C)[H])CO[C@]5([C@H]([C@@H]([C@@H]([C@H](O5)C)O)O)O)[H])C(O)=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('ClC1=C(O)C=C(O)C(=C1C)C(=O)O[C@H]2[C@@]3(O)C(=C[C@@H]4CC(C[C@@H]4[C@@]3(C)C2)(C)C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O1[C@@H](O[C@@H]([C@@H](O)[C@H](O)CO)[C@@H](N)C=O)[C@H](O)[C@@H](O)[C@H](O)[C@H]1C(O)=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), ('[H]C(=O)CCCC([O-])=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('FC1=CC=C(C=2N(C(C(C)C)=C(C2C3=CC=CC=C3)C(=O)NC4=CC=CC=C4)CCC=O)C=C1', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,N; C(=O) "
               "bonded to C,N; C(=O) bonded to C'), "
               "('[H]C(=O)C1=CC=C(C=C1)C([O-])=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) bonded to '
               "C'), "
               "('Cl[C@H]1[C@](C=C)([C@H]([N+]#[C-])[C@]2(O)C(=O)C=3C(NC=O)=CC=CC3C([C@H]2C1)(C)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to N'), "
               "('BrC1=C(O)C(=C(O)C=C1C)C(=O)C2=C(C(O)=CC(=C2CC=C(C)C)O)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('OC1C(C(CC/C=C(\\\\C)/C=O)C)=CC(=O)C(=C1)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to '
               "C,C; C(=O) bonded to C,C'), "
               "('CNC[C@@H]1CC[C@@H](N)[C@@H](O[C@@H]2[C@H](N)C[C@H](OC)[C@H]([C@H]2O)N(C)C(=O)CNC=O)O1', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to N'), "
               "('CC1(C)C2CCC1(C=O)C(=O)C2', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C,C; C(=O) bonded '
               "to C,C'), "
               "('O[C@@H]1[C@@H]([C@H](O[C@@H]([C@H]1NC([H])=O)C)O[C@@H]2[C@@H]([C@H](O[C@@H]([C@H]2NC([H])=O)C)O[C@@H]3[C@H](O[C@@H]([C@H]([C@@H]3O)NC([H])=O)C)*)O)O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N; C(=O) "
               "bonded to N; C(=O) bonded to N'), "
               "('[H][C@@]12C[C@@H](CC[C@@]1([H])[C@]1(C)CC[C@@H](O)C(C)(C)[C@@]1([H])[C@H](O)C2=O)C(=C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C(O)C1=C(O)C(=C(O)C=C1C)C=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) bonded to '
               "C'), ('[H]C(=O)C1C(=O)Cc2ccccc12', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C,C; C(=O) bonded '
               "to C,C'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CC[C@@]4([H])C(=O)C(NC=O)=CC[C@]4(C)[C@@]3([H])CC[C@]12C)[C@H](C)N(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to N'), ('[O-]C(=O)[*]C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to *; C(=O) "
               "bonded to *; C(=O) bonded to *'), "
               "('O=C(N[C@H]([C@@H](O)C=O)[C@@H](O)[C@H](O)C)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) bonded to '
               "N,C; C(=O) bonded to C'), ('OC(\\\\C=C\\\\C([O-])=O)=C/C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O1C(OC2=C(CCC=O)C=CC=C2O)C(O)C(O)C(O)C1C(O)=O', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('O=C(C1=C(C(O)=CC=C1CC=C(C)C)C=O)C2=C(O)C(OC[C@H](O)C(O)(C)C)=CC(=C2)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('C1CC(C(C(=O)[H])OC1)OC(C)=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) bonded to '
               "C'), ('CC(C)(C=O)[C@@H](O)C([O-])=O', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), "
               "('CSCC[C@H](NC=O)C(=O)O[C@@H]1[C@@H](COP([O-])(-*)=O)O[C@H]([C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C(O)CC(NC1=CC=C(C=O)C=C1)C(O)/C=C/C(O)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('ClC1=C(OC)C=C(O)C(=C1C)C(=O)O[C@H]2[C@@]3(OC)C(=C[C@@H]4CC(C[C@@H]4[C@@]3(C)C2)(C)C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C1O[C@@H](OC)C2=C1C=C(O)C(=C2O)C=O', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('O=C1OC(=C(C=O)C(=C1C)OC)/C=C/C(=O)OC', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), ('O=C(N)[C@@H](NC=O)CCCCN', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to N'), "
               "('O=C1OC(=C)[C@H](C)C=2C1=C(O)C(=C(O)C2C=O)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), ('O=C1OCC2=CC=C(N2[C@H]1CC(C)C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O([C@@H]1[C@](C2[C@@](C3[C@]([C@]4(C(C5[C@@](CC4)(CCC(C5)(C)C=O)C(O[C@@H]6O[C@@H]([C@@H](O)[C@H](O)[C@H]6O)CO)=O)=CC3)C)(CC2)C)(C[C@@H]1O)C)(CO)C)[C@@H]7OC[C@@H](O)[C@H](O)[C@H]7O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('N([C@H]1C[C@@H]([C@H](O1)CO*)O*)C=2NC(NC(C2N(C=O)C)=O)N', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to N'), "
               "('O=C([C@H](C1=C(C(O)=C(C)C(=C1C)O)C=O)C)C', 'Contains exactly "
               '3 ketone groups: C(=O) bonded to C,C; C(=O) bonded to C,C; '
               "C(=O) bonded to C'), "
               "('O=C(O[C@@H]1[C@@]2(O)C(C[C@@H]2C(=C[C@@H](C=C(C1)C)OC)C=O)(C)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C(OC)C1=C(O)C(=C(O)C=C1C)C=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) bonded to '
               "C'), "
               "('O=C1O[C@@H](CC=CC=CC(OC2OC(C(N(C)C)CC2)C)[C@@H](C[C@@H]([C@@H]([C@H](C(C1)O)OC)OC3OC(C(OC4OC(C(O)C(C4)(O)C)C)C(C3O)N(C)C)C)C(C=O)=C)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('Cl.N1=C(N=CC(=C1N)CN(C=O)/C(=C/2\\\\SC(OCC2)=O)/C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N; C(=O) "
               "bonded to S; C(=O) bonded to S'), "
               "('[H]C(=O)COCCOc1ccc(Nc2c(Cl)cccc2Cl)c(CC(O)=O)c1', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('OC1(C2CC3(C(C4(C(CC3)C(CCC4)(C)C=O)C)CC2)C1)COC(=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('[NH3+]\\\\C(=C\\\\C=C/C=O)C([O-])=O', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('O=C1C2=C(C=C(C)C=C2[C@H](C1)OC)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O[C@@H]1[C@]2([C@]([C@]3(C(C1)(C(CC3)C(=O)CO)C=O)[H])(CC[C@]4([C@@]2(CC[C@@H](O)C4)C)[H])[H])[H]', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O(C(=O)CCCCCCC/C=C\\\\CCCCCCCC)CC=1C(=C(O)C(C\\\\C=C(\\\\CCC=C(C)C)/C)=C(OC)C1)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C1C=C(C(C)C)[C@@H]2[C@]1(CC[C@@]3(C2=CC=C(C=O)C[C@@H]3O)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C1O[C@H](O)C=2C1=C(OC)C(=C(O)C2C=O)C', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('OC(=O)CCCCCCCCCCCCCCCCCCCCCCCCC=O', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('O=C/1C2=C(C=CC(=C2)O)O\\\\C1=C/C=C/C=C(/C=O)\\\\C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C1O[C@H]([C@@H](O)C=CC=2C(=C(O)C=CC2OC3=CC1=C(C=C3)C)C=O)CCCCC', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O(C1C(C2C(C(C(O)(CC2)C)C=O)(CC1)C)(C)C)C(=O)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('O=C(C1=C(O)C(=C(O)C(=C1)C(OC)C(C)C)C=O)C([C@@H]2O[C@@H](O)[C@]3(O[C@H]3CC)[C@@H](C2)O)C(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C1[C@]2([C@@H](C=O)[C@]3([C@@]1(CC[C@@H](C3)O)C)O)C[C@@]4([C@@H]([C@@H](/C=C/C(C(C)C)C)C)CCC4C2)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C1[C@@H]2C(=CC[C@H]3/C(=C(/C=C/C=C(C)C)\\\\C)/CC[C@@]3(C[C@@H]2[C@@](C1)(O)C)C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('[H][C@@](O)(CO)[C@@]([H])(O)[C@]([H])(O)[C@]([H])(NC(=O)CO)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to C'), "
               "('O1C(C(O)C(O)C(O)C1OC2=C(OC)C=C(C=C2)C=3OC=4C(C(=O)C3)=C(O)C(OC)=C(O)C4OC)COC(=O)CC(O)(COC=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to '), "
               "('COc1c(C)c2O[C@@H](CC(=O)c2c(O)c1C=O)c1ccccc1', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) bonded to '
               "C,C; C(=O) bonded to C'), "
               "('O=C(O)/C=C/C=C/C=C/[C@@H]1[C@@](O)(C(=C[C@H]2[C@H]1CC=C(C2)C=O)C)[C@]3(O[C@@H]3[C@@H](CC)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O1[C@@H](O[C@@H]([C@H](O)[C@H](O)CO)[C@@H](NC(=O)C)C=O)[C@@H](O)[C@H](O)[C@H](O)[C@@H]1C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to C'), "
               "('C[C@]12CC[C@]3([C@]([C@@]1(CC[C@]2(C)O)[H])(CC[C@@]4([C@@]3(CC(C(=O)C4)C=O)C)[H])[H])[H]', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O[C@H](C=O)C(=O)COP(O)(O)=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C,C; C(=O) bonded '
               "to C,C'), "
               "('O1C(C(O)C(O)C1N2C3=NC(=NC(N)=C3N=C2)NCCC4=CC=C(C=C4)CCC(O)=O)C(O)N(CC)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to N'), "
               "('O=C(C=1O[C@H]2[C@H](O)[C@@](O)(C)CC[C@H]2C1C=O)C(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O(C\\\\C=C(\\\\CC\\\\C=C(\\\\CC\\\\C=C(\\\\CC/C=C(\\\\C)/C=O)/C)/C)/C)C(=O)/C(/C)=C/O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C1O[C@H](OC)C=2C1=C(OC)C(=C(OC[C@H]3[C@](O)(CC[C@@H]4[C@@]3(CCCC4(C)C)C)C)C2C=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('[C@H]1([C@H]([C@H]([C@@H]([C@H](O1)C)NC([H])=O)O)O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)C)*)O)O)O[C@@H]3[C@@H]([C@H](O[C@@H]([C@H]3NC([H])=O)C)O[C@@H]4[C@H](O[C@@H]([C@H]([C@@H]4O)NC([H])=O)C)OC)O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N; C(=O) "
               "bonded to N; C(=O) bonded to N'), "
               "('CC1(C)[C@H](CC[C@@]2(C)[C@H]1CC[C@]1(C)[C@@H]2C[C@H]2O[C@]22[C@@H]3C[C@@](C)(C=O)[C@H](C[C@]3(C)[C@@H](O)C[C@@]12C)OC(=O)c1ccccc1)O[C@@H]1OC[C@H](O[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('[H][C@]12CCNC[C@]1(OC[C@@]2(C(=O)OC)c1[nH]c2ccccc2c1C=O)C=C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('[H][C@@]12[C@@H](C[C@@H](C)[C@]3(CC\\\\C(O3)=C/C=O)[C@@]1(C)CCCC2(C)C)OC(C)=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C(O[C@H]1C2=C(C=O)[C@@H](O)[C@@H]3CC(C[C@@H]3[C@@]2(C)C1)(C)C)C4=C(O)C=C(O)C=C4C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C(CC1=C(C(O)=C(C)C(=C1)O)C=O)C[C@@H](OC)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) bonded to '
               "C,C; C(=O) bonded to C'), "
               "('C1CC([C@]2([C@](C1)([C@@]3([C@@](CC2)([C@]4([C@]([C@@H](C3)OC(=O)C)(CC(=CC4)C(=O)[H])C)[H])C)[H])C)[H])(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C(NNC1=CC(=C(O)C=C1)C=O)C', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to N,C; C(=O) bonded to N,C; C(=O) bonded '
               "to C'), "
               "('[H]C(=O)[C@@]1(O)CCC(=O)[C@@H](OC)[C@]1([H])[C@@]1(C)O[C@@H]1CC=C(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C,C; C(=O) bonded to C,C'), "
               "('[H]C(=O)C(\\\\C)=C(/O)\\\\C=C(/O)C(O)=O', 'Contains exactly "
               '3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('OC(CCCC/C(=N\\\\CC(O)=O)/C=O)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('O=C(N1C2N3C(C=4C=CC=C5C4C6(C2(C7=C(NC6N5C=O)C=CC=C7)CC1)CC3)C8OC8(C)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to N'), "
               "('[H]C(=O)\\\\C=C(C)/C=C/[C@@]1(O)C(C)=CC(=O)CC1(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C,C; C(=O) bonded to C,C'), "
               "('O=C1OC2=C(C(=CC(=C2C)O)C)OC3=C1C(=CC(=C3C=O)O)C', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('ClC1=C(O)C(=C2OC=3C(=CC(=C(C3OC(C2=C1C)=O)C)OC)C)C(=O)[H]', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=CC12C(C(C(CC1)C)(CC(O)=O)C)CCC=C2CO', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), "
               "('[H]C(=O)C1=C/C[C@@]2([H])[C@]([H])(CC[C@]2(C)C[C@@]2([H])[C@]\\\\1([H])C(=O)C[C@@]2(C)O)[C@@H](C)CCC=C(C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C,C; C(=O) bonded to C,C'), "
               "('S(O[C@H]([C@H](O)[C@@H](NC(=O)C)C=O)[C@H](O)CO)(O)(=O)=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to N,C; C(=O) "
               "bonded to N,C; C(=O) bonded to C'), "
               "('ClC1=C(OC)C(=C2OC(=O)C3=C(OC2=C1C)C(=C(O)C(=C3C)Cl)C=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('N[C@@H](\\\\C=C\\\\ONC=O)C(O)=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to N; C(=O) bonded to C; C(=O) bonded to '
               "C'), "
               "('O=C(O)[C@@H]([C@@H]1[C@@]2([C@@](C3=C([C@@]4([C@H](C([C@@H](O)CC4)(C)C)CC3)C)CC2)(C)CC1)C)CC/C=C(/C=O)\\\\C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('O=C1[C@@H]2C(=CC[C@H]3C([C@H](/C=C\\\\C=C(C)C)C)CC[C@@]3(C[C@@H]2[C@@](C1)(O)C)C)C=O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('C=1C2=C(C=CC1)NC(C(C2=O)C)C([H])=O', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C,C; C(=O) bonded to C,C; C(=O) '
               "bonded to C'), ('C1(=CN=C(C(=C1C(=O)[O-])O)C)C=O', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C; C(=O) bonded to C; '
               "C(=O) bonded to C'), "
               "('O=C(O[C@H]1[C@H](O)C([C@@H]2CC[C@H]([C@]3([C@]2(C1)C)OC=4C=C(CO)C(=C(C4C3)O)C=O)C)(C)C)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('OC(CCCCC)/C(=N\\\\CC(O)=O)/C=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) bonded to '
               "C'), "
               "('OC[C@H]1O[C@@H](O[C@H]([C@@H](O)C=O)[C@H](O)[C@H](O)C(O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('[H][C@]12C[C@]3(C)C([C@@H](O)C1=C(C)C(=O)O2)=C(C=O)[C@@]1([H])C[C@@]31[H]', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C'), "
               "('N1(CCN(CC1)C(=O)C(Cl)Cl)C=O', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to N,C; C(=O) bonded to N,C; C(=O) bonded '
               "to N'), ('O=C1OCC=2C1=C(OC)C(=CC2C)C=O', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('[H]C(=O)C(=O)O[*]', 'Contains exactly 3 "
               'ketone groups: C(=O) bonded to C; C(=O) bonded to C; C(=O) '
               "bonded to C'), ('O[C@H]([C@H](O)C)[C@H](O)C(=O)C=O', 'Contains "
               'exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) bonded to '
               "C,C; C(=O) bonded to C'), "
               "('O=C1C2=C(O)C(O)=C(O)C(=C2[C@H]3OC4=C([C@@H]1O3)C(=C(C)C(=C4O)O)C=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C(NCCCCCN1C(=CC=C1CO)C=O)C', 'Contains exactly 3 ketone "
               'groups: C(=O) bonded to N,C; C(=O) bonded to N,C; C(=O) bonded '
               "to C'), "
               "('O=C(C1=C(C(O)=CC=C1CC=C(C)C)C=O)C2=C(O)C(OC[C@@H]3OC3(C)C)=CC(=C2)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C,C; C(=O) "
               "bonded to C,C; C(=O) bonded to C'), "
               "('O=C\\\\C(=C\\\\CC1C(CCCC1=C)(C)C)\\\\C=C\\\\OC(=O)C', "
               "'Contains exactly 3 ketone groups: C(=O) bonded to C; C(=O) "
               "bonded to C; C(=O) bonded to C')]\n"
               'False negatives: '
               "[('[C@@H]1([C@H](CC[C@]2(O1)C[C@@H]3OC(C=C[C@H](C)[C@@H](O)[C@H](C)C(=O)[C@H](C)[C@@H](O)[C@H](C)C([C@@]([C@@H](O)[C@H](C)CC=CC=C[C@@H](CC[C@@H]([C@H]3C)O2)CC)(O)C)=O)=O)C)C[C@H](O)C', "
               "'Contains 6 ketone groups (more than 3)'), "
               "('[O-][N+](=O)c1cc(ccc1C(=O)C1C(=O)CCCC1=O)C(F)(F)F', "
               "'Contains 6 ketone groups (more than 3)')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 14331,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 0.5,
    'f1': 0.019417475728155338,
    'accuracy': 0.9930021478556087}


"""
Classifies: CHEBI:132130 hydroxyquinone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMolFrags

def is_hydroxyquinone(smiles: str):
    """
    Determines if a molecule is a hydroxyquinone (quinone with one or more hydroxy substituents
    on the quinone ring system).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxyquinone, False otherwise
        str: Reason for classification
    """
    # Convert SMILES to molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find quinone patterns
    # Pattern 1: para-quinone
    patt1 = Chem.MolFromSmarts('[#6]=O.[#6]=O.[$([#6]1=,:[#6][#6]=,:[#6][#6]=,:[#6]1),$([#6]1=,:[#6][#6]=,:[#6][#6]=,:[#6][#6]1)]')
    # Pattern 2: ortho-quinone
    patt2 = Chem.MolFromSmarts('[#6](=O)-[#6]=,:[#6]-[#6](=O)')
    
    matches1 = mol.GetSubstructMatches(patt1)
    matches2 = mol.GetSubstructMatches(patt2)
    
    if not (matches1 or matches2):
        return False, "No quinone moiety found"

    # Find hydroxy groups
    oh_pattern = Chem.MolFromSmarts('[OH]')
    oh_matches = mol.GetSubstructMatches(oh_pattern)
    
    if not oh_matches:
        return False, "No hydroxy groups found"

    # For each quinone pattern found, check if any hydroxy groups are attached to the quinone ring
    quinone_atoms = set()
    for match in matches1:
        quinone_atoms.update(match)
    for match in matches2:
        quinone_atoms.update(match)

    # Check if any OH groups are connected to quinone ring atoms
    for oh_match in oh_matches:
        oh_atom = mol.GetAtomWithIdx(oh_match[0])
        for neighbor in oh_atom.GetNeighbors():
            if neighbor.GetIdx() in quinone_atoms:
                num_oh = len([m for m in oh_matches if mol.GetAtomWithIdx(m[0]).GetNeighbors()[0].GetIdx() in quinone_atoms])
                return True, f"Found hydroxyquinone with {num_oh} hydroxy group(s) on quinone ring"

    return False, "No hydroxy groups attached to quinone ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:132130',
                          'name': 'hydroxyquinone',
                          'definition': 'Any quinone in which one or more of '
                                        'the carbons making up the quinone '
                                        'moiety is substituted by a hydroxy '
                                        'group.',
                          'parents': ['CHEBI:33822', 'CHEBI:36141']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 814,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.8914223669923995}


"""
Classifies: CHEBI:28034 beta-D-galactoside
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_beta_D_galactoside(smiles: str):
    """
    Determines if a molecule is a beta-D-galactoside, defined as any D-galactoside having
    beta-configuration at its anomeric centre.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a beta-D-galactoside, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the galactose substructure with beta configuration at the anomeric centre
    pat = Chem.MolFromSmarts('O[C@@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@@H]1O')
    if mol.HasSubstructMatch(pat):
        return True, "Molecule contains a beta-D-galactoside substructure"
    else:
        return False, "Molecule does not contain a beta-D-galactoside substructure"

# Examples
print(is_beta_D_galactoside('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CC=C4C[C@H](CC[C@]4(C)[C@@]3([H])CC[C@]12C)O[C@@H]1O[C@H](COC(C)=O)[C@H](O)[C@H](O)[C@H]1O)[C@H](C)CCCC(C)C'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')

print(is_beta_D_galactoside('O(C=1C=C2OC(=O)C=C(C2=CC1)C)[C@H]3[C@@H]([C@@H](O)[C@H]([C@H](O3)CO)O)O'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')

print(is_beta_D_galactoside('O[C@H]1[C@@H](O)[C@@H](COP(O)(O)=O)O[C@@H](O[*])[C@@H]1O'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')

print(is_beta_D_galactoside('OC[C@H]1O[C@@H](Oc2cc3c(O)cc(O)cc3[o+]c2-c2ccc(O)c(O)c2)[C@H](O)[C@@H](O)[C@H]1O'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')

print(is_beta_D_galactoside('COc1cc(ccc1O)-c1oc2cc(O)cc(O)c2c(=O)c1O[C@@H]1O[C@H](CO)[C@H](O)[C@H](O)[C@H]1O'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')

print(is_beta_D_galactoside('[C@@H]1([C@@H](O[C@@H]([C@@H]([C@@H]1O)O)CO)O*)O[C@H]2[C@H]([C@@H]([C@@H]([C@@H](O2)C)O)O)O'))
# Output: (True, 'Molecule contains a beta-D-galactoside substructure')


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28034',
                          'name': 'beta-D-galactoside',
                          'definition': 'Any D-galactoside having '
                                        'beta-configuration at its anomeric '
                                        'centre.',
                          'parents': ['CHEBI:20961']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 14196,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9925884491679485}


"""
Classifies: CHEBI:17088 monoacyl-sn-glycerol 3-phosphate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_monoacyl_sn_glycerol_3_phosphate(smiles: str):
    """
    Determines if a molecule is a monoacyl-sn-glycerol 3-phosphate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a monoacyl-sn-glycerol 3-phosphate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for phosphate group
    patt_phosphate = Chem.MolFromSmarts('[PX4](=O)([OH])[OH]')
    if not mol.HasSubstructMatch(patt_phosphate):
        return False, "No phosphate group found"
        
    # Check for glycerol backbone with one acyl group
    # Match pattern: -O-CH2-CH(OH)-CH2-O-P where one of the non-phosphate oxygens has an acyl group
    patt_glycerol = Chem.MolFromSmarts('[OX2][CH2][CH]([OH])[CH2][OX2]P(=O)([OH])[OH]')
    if not mol.HasSubstructMatch(patt_glycerol):
        return False, "No glycerol backbone with phosphate found"
        
    # Check for exactly one acyl group (-C(=O)-)
    patt_acyl = Chem.MolFromSmarts('[CX3](=O)')
    acyl_matches = mol.GetSubstructMatches(patt_acyl)
    
    if len(acyl_matches) == 0:
        return False, "No acyl group found"
    elif len(acyl_matches) > 1:
        return False, "More than one acyl group found"
        
    # Check if acyl group is attached to glycerol backbone
    patt_full = Chem.MolFromSmarts('[CX3](=O)-[OX2]-[CH2]-[CH]([OH])-[CH2]-[OX2]-P(=O)([OH])[OH]')
    patt_full2 = Chem.MolFromSmarts('[CX3](=O)-[OX2]-[CH2]-[CH](-[OX2]-[CH2]-P(=O)([OH])[OH])[OH]')
    
    if not (mol.HasSubstructMatch(patt_full) or mol.HasSubstructMatch(patt_full2)):
        return False, "Acyl group not properly connected to glycerol backbone"

    return True, "Molecule is a monoacyl-sn-glycerol 3-phosphate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17088',
                          'name': 'monoacyl-sn-glycerol 3-phosphate',
                          'definition': 'An sn-glycero-3-phosphate compound '
                                        'having a single unspecified acyl '
                                        'group at either position 1 or '
                                        'position 2.',
                          'parents': ['CHEBI:16961', 'CHEBI:26706']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 5,
    'num_true_negatives': 183864,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.4444444444444444,
    'recall': 0.6666666666666666,
    'f1': 0.5333333333333333,
    'accuracy': 0.9999619306594154}


"""
Classifies: CHEBI:16179 retinyl ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_retinyl_ester(smiles: str):
    """
    Determines if a molecule is a retinyl ester.

    A retinyl ester is a carboxylic ester obtained by formal condensation of
    the hydroxy group of retinol with the carboxy group of any carboxylic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a retinyl ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a retinol substructure
    retinol_smarts = 'C(=C)C=C(/C=C/C=C(C)/C=C/C(C)(C)CCCC(C)(C)O)'
    retinol_mol = Chem.MolFromSmarts(retinol_smarts)
    is_retinol = mol.HasSubstructMatch(retinol_mol)

    # Check for the presence of an ester group
    ester_smarts = 'C(=O)OC'
    ester_mol = Chem.MolFromSmarts(ester_smarts)
    is_ester = mol.HasSubstructMatch(ester_mol)

    if is_retinol and is_ester:
        return True, "Molecule is a retinyl ester"
    elif is_retinol:
        return False, "Molecule contains a retinol substructure but is not an ester"
    elif is_ester:
        return False, "Molecule is an ester but does not contain a retinol substructure"
    else:
        return False, "Molecule is neither a retinyl ester nor contains a retinol substructure or ester group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16179',
                          'name': 'retinyl ester',
                          'definition': 'A carboxylic ester obtained by formal '
                                        'condensation of the hydroxy group of '
                                        'retinol with the carboxy group of any '
                                        'carboxylic acid.',
                          'parents': ['CHEBI:26537', 'CHEBI:33308']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183919,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945628534145}


"""
Classifies: CHEBI:32878 alkene
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_alkene(smiles: str):
    """
    Determines if a molecule is an alkene.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an alkene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check if the molecule contains only C and H atoms
    atom_symbols = set(atom.GetSymbol() for atom in mol.GetAtoms())
    if atom_symbols != {'C', 'H'}:
        return False, "Molecule contains non-hydrocarbon atoms"
    
    # Count the number of double bonds
    double_bonds = sum(bond.GetBondType() == Chem.BondType.DOUBLE for bond in mol.GetBonds())
    
    # Check for the presence of at least one double bond
    if double_bonds == 0:
        return False, "Molecule does not contain any double bonds"
    elif double_bonds > 1:
        return False, "Molecule contains more than one double bond"
    
    # Check if the molecule is acyclic
    if Chem.Descriptors.IsRingPresent(mol):
        return False, "Molecule is cyclic"
    
    return True, "Molecule is an alkene"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:32878',
                          'name': 'alkene',
                          'definition': 'An acyclic branched or unbranched '
                                        'hydrocarbon having one carbon-carbon '
                                        'double bond and the general formula '
                                        'CnH2n. Acyclic branched or unbranched '
                                        'hydrocarbons having more than one '
                                        'double bond are alkadienes, '
                                        'alkatrienes, etc.',
                          'parents': ['CHEBI:33645']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183869,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673691366417}